﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

using HtmlAgilityPack;

namespace mvcForum.AddOns {

	public static class HTMLSanitizer {

		public static String SanitizeHTML(this String html) {
			return html.SanitizeHTML(new String[] { });
		}

		public static String SanitizeHTML(this String html, String[] whitelist) {
			HtmlDocument doc = new HtmlDocument();
			doc.LoadHtml(html);

			if (!whitelist.Contains("#text")) {
				whitelist = whitelist.Concat(new String[] { "#text" }).ToArray();
			}

			StringWriter sw = new StringWriter();
			ConvertTo(doc.DocumentNode, sw, whitelist);
			sw.Flush();
			return sw.ToString();
		}


		private static void ConvertContentTo(HtmlNode node, TextWriter outText, String[] whitelist) {
			foreach (HtmlNode subnode in node.ChildNodes) {
				ConvertTo(subnode, outText, whitelist);
			}
		}

		public static void ConvertTo(HtmlNode node, TextWriter outText, String[] whitelist) {
			String html;
			switch (node.NodeType) {
				case HtmlNodeType.Comment:
					// We're not going to output comments!
					break;

				case HtmlNodeType.Document:
					ConvertContentTo(node, outText, whitelist);
					break;

				case HtmlNodeType.Text:
					html = ((HtmlTextNode)node).Text;

					// Is it in fact a special closing node output as text?
					if (HtmlNode.IsOverlappedClosingElement(html)) {
						break;
					}

					// Do we actually have any output??
					if (!String.IsNullOrWhiteSpace(html)) {
						outText.Write(HtmlEntity.DeEntitize(html));
					}
					break;

				case HtmlNodeType.Element:
					// Is this tag in the white-list? Then we'll output the start tag!
					if (whitelist.Contains(node.Name.ToLower())) {
						outText.Write(String.Format("<{0}>", node.Name));
					}

					if (node.HasChildNodes) {
						ConvertContentTo(node, outText, whitelist);
					}

					// Is this tag in the white-list? Then we'll output the end tag!
					if (whitelist.Contains(node.Name.ToLower())) {
						outText.Write(String.Format("</{0}>", node.Name));
					}
					break;
			}
		}
	}
}