﻿/* mvcForum
 * Copyright (C) 2010-2011 Steen F. Tøttrup
 * http://mvcforum.org/
 */

using System;
using System.ComponentModel.DataAnnotations;

namespace mvcForum.Core {

	/// <summary>
	/// This class represent the timestamp of when a user last viewed a topic.
	/// </summary>
	public class TopicTrack {
		public TopicTrack() { }
		public TopicTrack(Topic topic, ForumUser user) {
			this.Topic = topic;
			this.ForumUser = user;
			this.LastViewed = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
		}

		public virtual void Viewed() {
			this.LastViewed = DateTime.UtcNow;
		}

		#region Properties
		public Int32 Id { get; set; }
		/// <summary>
		/// The timestamp indicating when the user last viewed the topic.
		/// </summary>
		[Required]
		public DateTime LastViewed { get; set; }
		/// <summary>
		/// The topic tracked.
		/// </summary>
		[Required]
		public Int32 TopicId { get; set; }
		public virtual Topic Topic { get; set; }
		/// <summary>
		/// The user.
		/// </summary>
		[Required]
		public Int32 ForumUserId { get; set; }
		public virtual ForumUser ForumUser { get; set; }
		#endregion
	}
}