﻿using System;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;

using ApplicationBoilerplate.DataProvider;

using MVCBootstrap;
using MVCBootstrap.Web.Mvc.Interfaces;

using mvcForum.Core;
using mvcForum.Core.Abstractions.Interfaces;
using mvcForum.Core.Specifications;
using mvcForum.Web.Controllers;
using mvcForum.Web.Filters;
using mvcForum.Web.Helpers;
using mvcForum.Web.Interfaces;
using mvcForum.Web.Services;
using mvcForum.Web.ViewModels;
using System.Net.Mail;
using CreativeMinds.Security.Cryptography;

namespace mvcForum.Web.Areas.Forum.Controllers {

	[ConfigurationControlled(ConfigurationArea.AccountController, "")]
	public class AccountController : ThemedForumBaseController {
		private readonly IFormsAuthenticationService formsService;
		private readonly IMembershipService membershipService;
		private readonly IRepository<BannedIP> bipRepo;
		private readonly IConfiguration config;
		private readonly IMailService mailService;

		public AccountController(IContext context,
								IWebUserProvider userProvider,
								IFormsAuthenticationService formsService,
								IMembershipService membershipService,
								IRepository<BannedIP> bipRepo,
								IConfiguration config,
								IMailService mailService)
			: base(userProvider, context) {

			this.formsService = formsService;
			this.membershipService = membershipService;
			this.config = config;
			this.bipRepo = bipRepo;
			this.mailService = mailService;
		}

		public ActionResult LogOff() {
			this.formsService.SignOut();
			return RedirectToAction("index", "home");
		}

		[HttpGet]
		[ConfigurationControlled(ConfigurationArea.AccountController, "SignUp")]
		public ActionResult Register() {
			if (!this.config.UseForumAccountController || !this.config.AllowLocalUsers || !this.config.AllowSignUp) {
				return new HttpNotFoundResult();
			}
			return View(new RegisterModel {
				RequireRulesAccept = this.config.RequireRulesAccept,
				SignUpRules = this.config.SignUpRules
			});
		}

		[HttpPost]
		[ConfigurationControlled(ConfigurationArea.AccountController, "SignUp")]
		public ActionResult Register(RegisterModel model) {
			if (!this.config.UseForumAccountController || !this.config.AllowLocalUsers || !this.config.AllowSignUp) {
				return new HttpNotFoundResult();
			}

			if (ModelState.IsValid) {
				if ((this.config.RequireRulesAccept && model.RulesAccepted) || !this.config.RequireRulesAccept) {
					// Attempt to register the user
					MembershipCreateStatus createStatus = this.membershipService.CreateUser(model.Username, model.Password, model.EmailAddress);
					if (createStatus == MembershipCreateStatus.Success) {
						if (this.config.RequireEmailValidation) {
							MembershipUser u = this.membershipService.GetUser(model.Username);

							String code = String.Format("{1}#|#{0}#|#{2}", u.UserName, u.Email, u.ProviderUserKey);

							PrivatePrivateCrypto crypt = new PrivatePrivateCrypto();
							// hm.....
							crypt.Phrase = this.config.DefaultTimezone;

							String siteUrl = this.config.SiteURL;
							if (!siteUrl.EndsWith("/")) {
								siteUrl += "/";
							}
							String url = String.Format("{0}forum/account/activate?code={1}", siteUrl, HttpUtility.UrlEncode(crypt.Encrypt(code)));

							this.mailService.Send(
								new MailAddress(this.config.RobotEmailAddress, this.config.RobotName),
								new MailAddress[] { new MailAddress(model.EmailAddress, model.Username) }.ToList(),
								this.config.ValidationSubject,
								this.config.ValidationBody
									.Replace("{Email}", model.EmailAddress)
									.Replace("{Password}", model.Password)
									.Replace("{Url}", url));

							TempData.Add("Status", ForumHelper.GetString<mvcForum.Web.ForumConfigurator>("Register.EmailActivation"));
						}
						else {
							IRepository<ForumUser> userRepo = this.Context.GetRepository<ForumUser>();
							ForumUser user = userRepo.ReadOne(new ForumUserSpecifications.SpecificEmailAddress(model.EmailAddress));
							user.Active = true;
							this.Context.SaveChanges();

							TempData.Add("Status", ForumHelper.GetString<mvcForum.Web.ForumConfigurator>("Register.AccountReady"));
						}

						return RedirectToAction("success", "account", new { area = "forum" });
					}
					else {
						ModelState.AddModelError("", this.membershipService.ErrorCodeToString(createStatus, this.textManager));
					}
				}
				else {
					ModelState.AddModelError("RulesAccepted", ForumHelper.GetString<mvcForum.Web.ForumConfigurator>("Register.RulesMustBeAccepted"));
				}
			}

			// If we got this far, something failed, redisplay form
			model.RequireRulesAccept = this.config.RequireRulesAccept;
			model.SignUpRules = this.config.SignUpRules;
			return View(model);
		}

		[HttpGet]
		[ConfigurationControlled(ConfigurationArea.AccountController, "LocalOrOpenAuth")]
		public ActionResult LogOn() {
			if (!this.config.UseForumAccountController || (!this.config.AllowLocalUsers && !this.config.AllowOpenAuthUsers)) {
				return new HttpNotFoundResult();
			}
			// Get any bans tied to this user's IP address.
			BannedIP banned = this.bipRepo.ReadOne(new BannedIPSpecifications.SpecificIP(this.Request.UserHostAddress));
			// Has the visitors IP Address been banned?
			if (banned != null) {
				// Yes, let's tell him!
				TempData.Add("Reason", ForumHelper.GetString<mvcForum.Web.ForumConfigurator>("NoAccess.BannedIP", new { IPAddress = Request.UserHostAddress }));
				// Off to the "no access" page.
				return RedirectToRoute("NoAccess");
			}

			return View(new LogOnModel { AllowLocalUsers = this.config.AllowLocalUsers, AllowSignUp = this.config.AllowSignUp, AllowOpenAuthUsers = this.config.AllowOpenAuthUsers });
		}

		[HttpPost]
		[ConfigurationControlled(ConfigurationArea.AccountController, "Local")]
		public ActionResult LogOn(LogOnModel model, String returnUrl) {
			if (!this.config.UseForumAccountController || !this.config.AllowLocalUsers) {
				return new HttpNotFoundResult();
			}

			if (ModelState.IsValid) {
				BannedIP banned = this.bipRepo.ReadOne(new BannedIPSpecifications.SpecificIP(this.Request.UserHostAddress));
				// Found any?
				if (banned != null) {
					// The IP address is banned!
					TempData.Add("Reason", ForumHelper.GetString<mvcForum.Web.ForumConfigurator>("NoAccess.BannedIP"));
					// Off to the "no access" page.
					return RedirectToRoute("NoAccess");
				}

				String username = this.membershipService.GetUsername(model.EmailAddress);
				// Let's try to validate the user.
				if (!String.IsNullOrWhiteSpace(username) && this.membershipService.ValidateUser(username, model.Password)) {
					// Success, let's sign the user in.
					ForumUser u = this.ForumUserRepository.ReadOne(new ForumUserSpecifications.SpecificUsername(username));
					if (u != null) {
						if (!u.ExternalAccount) {
							this.formsService.SignIn(username, model.RememberMe);

							if (Url.IsLocalUrl(returnUrl)) {
								return Redirect(returnUrl);
							}
							else {
								return RedirectToAction("Index", "Home", new { area = "forum" });
							}
						}
						else {
							// External (OpenAuth account), should not log on through this action!!
							// TODO: Get error message form resources
							ModelState.AddModelError(String.Empty, "The account is an external account, please log on using the right provider.");
						}
					}
				}
				else {
					// Couldn't validate the user!
					// TODO: Get error message form resources
					ModelState.AddModelError(String.Empty, "The user name or password provided is incorrect.");
				}
			}

			model.AllowLocalUsers = this.config.AllowLocalUsers;
			model.AllowSignUp = this.config.AllowSignUp;
			model.AllowOpenAuthUsers = this.config.AllowOpenAuthUsers;
			return View(model);
		}

		public ActionResult External() {
			return View();
		}

		public ActionResult Success() {
			return View();
		}

		[ConfigurationControlled(ConfigurationArea.AccountController, "SignUp")]
		public ActionResult Activate(String code) {
			Boolean success = false;
			if (!String.IsNullOrWhiteSpace(code)) {
				PrivatePrivateCrypto crypt = new PrivatePrivateCrypto();
				// Hm...
				crypt.Phrase = this.config.DefaultTimezone;
				try {
					String decrypted = crypt.Decrypt(code);

					String[] parts = decrypted.Split(new String[] { "#|#" }, StringSplitOptions.RemoveEmptyEntries);
					if (parts.Length == 3) {
						MembershipUser u = this.membershipService.GetUser(parts[1]);
						if (u != null && u.Email == parts[0] && u.ProviderUserKey.ToString() == parts[2]) {
							u.IsApproved = true;
							this.membershipService.Update(u);
							success = true;

							ForumUser fu = this.ForumUserRepository.ReadOne(new ForumUserSpecifications.SpecificProviderUserKey(u.ProviderUserKey.ToString()));
							fu.Active = true;
							this.Context.SaveChanges();
						}
					}
				}
				catch {
				}
			}

			return View(success);
		}
	}
}