﻿using System;
using SimpleAuthentication.Mvc;
using System.Web.Mvc;
using System.Web;
using mvcForum.Web.ViewModels;
using MVCBootstrap.Web.Mvc.Interfaces;
using System.Web.Security;
using System.Net.Mail;
using mvcForum.Core;
using ApplicationBoilerplate.DataProvider;
using mvcForum.Core.Specifications;

namespace mvcForum.Web.Areas.Forum.Controllers {

	public class AuthController : IAuthenticationCallbackProvider {
		private readonly IMembershipService memberService;
		private readonly IFormsAuthenticationService forms;
		private readonly IContext context;

		public AuthController(IMembershipService memberService,
								IContext context,
								IFormsAuthenticationService forms) {
			this.memberService = memberService;
			this.context = context;
			this.forms = forms;
		}

		public ActionResult OnRedirectToAuthenticationProviderError(HttpContextBase context, String errorMessage) {
			return new ViewResult {
				ViewName = "AuthenticateCallback",
				ViewData = new ViewDataDictionary(new {
					ErrorMessage = errorMessage
				})
			};
		}

		public ActionResult Process(HttpContextBase context, AuthenticateCallbackData model) {
			//String userKey = String.Format("{0}-{1}", model.AuthenticatedClient.ProviderName, model.AuthenticatedClient.UserInformation.Id);
			MembershipUser u = this.memberService.GetUser(model.AuthenticatedClient.UserInformation.UserName);
			if (u == null) {
				// New external user, let's create the user etc.
				MailAddress email = new MailAddress(String.Format("{0:ddMMyyyy-hhmmss}repl@ce.this", DateTime.UtcNow));
				try {
					email = new MailAddress(model.AuthenticatedClient.UserInformation.Email);
				}
				catch { }

				// Random password, and username from provider.
				MembershipCreateStatus status = this.memberService.CreateUser(model.AuthenticatedClient.UserInformation.UserName, new Guid().ToString(), email.Address, true);
				if (status == MembershipCreateStatus.Success) {
					u = this.memberService.GetUser(model.AuthenticatedClient.UserInformation.UserName);
					if (u != null) {
						IRepository<ForumUser> fuRepo = this.context.GetRepository<ForumUser>();
						ForumUser fu = fuRepo.ReadOne(new ForumUserSpecifications.SpecificProviderUserKey(u.ProviderUserKey.ToString()));
						if (fu != null) {
							fu.Name = model.AuthenticatedClient.UserInformation.UserName;
							fu.FullName = model.AuthenticatedClient.UserInformation.Name;
							fu.Active = true;
							fu.ExternalAccount = true;
							fu.ExternalProvider = model.AuthenticatedClient.ProviderName;
							fu.ExternalProviderId = model.AuthenticatedClient.UserInformation.Id;
							fuRepo.Update(fu);
							this.context.SaveChanges();
						}
					}
					else {
						// TODO:
						return new HttpStatusCodeResult(500);
					}
				}
				else {
					// TODO:
					return new HttpStatusCodeResult(500);
				}
			}
			else {
				// TODO: Validate same provider etc...!!
				IRepository<ForumUser> fuRepo = this.context.GetRepository<ForumUser>();
				ForumUser fu = fuRepo.ReadOne(new ForumUserSpecifications.SpecificProviderUserKey(u.ProviderUserKey.ToString()));
				if (fu != null) {
					// Not an external account? Or external, but with a different provider?
					if (!fu.ExternalAccount || (fu.ExternalAccount && fu.ExternalProvider != model.AuthenticatedClient.ProviderName)) {
						// TODO: ????
						return new RedirectResult("/forum/account/external");
					}
				}
				else {
					// TODO:
					return new HttpStatusCodeResult(500);
				}
			}

			this.forms.SignIn(model.AuthenticatedClient.UserInformation.UserName, false);

			return new RedirectResult("/forum");
		}
	}
}