﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;

using ApplicationBoilerplate.DataProvider;

using MVCBootstrap;

using mvcForum.Core;
using mvcForum.Core.Interfaces;
using mvcForum.Web.Attributes;
using mvcForum.Web.Controllers;
using mvcForum.Web.Extensions;
using mvcForum.Web.Helpers;
using mvcForum.Web.ViewModels;
using mvcForum.Web.Interfaces;

namespace mvcForum.Web.Areas.Forum.Controllers {

	public class FileController : ThemedForumBaseController {
		private readonly IAttachmentStorage storage;

		public FileController(IWebUserProvider userProvider, IContext context, IAttachmentStorage storage)
			: base(userProvider, context) {

			this.storage = storage;
		}

		[Authorize]
		public ActionResult Attach(Int32 id) {
			Post post = this.GetRepository<Post>().Read(id);

			// Is the user allowed to upload?
			if (post.Topic.Forum.HasAccess(AccessFlag.Upload)) {
				// For now, only the author can post files to a message!
				if (this.ActiveUser.Id == post.Author.Id) {
					Dictionary<String, String> path = new Dictionary<String, String>();
					// Build the breadcrumb path.
					HomeController.BuildPath(post.Topic, path, this.Url);
					return View(new MessageViewModel(post) { /*Message = message,*/ Path = path, Topic = new TopicViewModel(post.Topic, new MessageViewModel[] { }, 0, 1, false) /*{ Topic = message.Topic }*/ });
				}
			}

			// No, the user is not allowed to upload, let's inform him!
			TempData.Add("Reason", ForumHelper.GetString("NoAccess.NoUpload"));
			// Off to the "no access" page!
			return RedirectToRoute("NoAccess");
		}

		[HttpPost]
		[Authorize]
		[NotBanned]
		public ActionResult Attach(Int32 id, HttpPostedFileBase[] files) {
			// Get the post the file should be attached to.
			Post post = this.GetRepository<Post>().Read(id);

			// Is the user allowed to upload?
			if (post.Topic.Forum.HasAccess(AccessFlag.Upload)) {
				// For now, only the author can post files to a message!
				if (this.ActiveUser.Id == post.Author.Id) {
					foreach (var file in files) {
						// TODO: Right extension?!?!?!
						// TODO: Size!??!?!

						String extension = Path.GetExtension(file.FileName);
						if (extension.StartsWith(".")) {
							extension = extension.Substring(1);
						}

						// Create the new attachment.
						IRepository<Attachment> attachmentRepository = this.GetRepository<Attachment>();
						Attachment att = new Attachment(this.ActiveUser, post, "/attachments/", file.FileName, file.ContentLength);
						attachmentRepository.Create(att);

						att.Path = storage.CreateAttachment(att, file.InputStream, file.ContentType);
						this.Context.SaveChanges();
					}

					// File(s) uploaded, let's get back to the topic.
					return new RedirectToRouteResult("ShowTopic", new RouteValueDictionary { { "id", post.Topic.Id }, { "title", post.Topic.Title.ToSlug() } });
				}
			}

			// No, the user is not allowed to upload, let's inform him!
			TempData.Add("Reason", ForumHelper.GetString("NoAccess.NoUpload"));
			// Off to the "no access" page.
			return RedirectToRoute("NoAccess");
		}
	}
}