﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using System.Web.Security;

using ApplicationBoilerplate.DataProvider;

using MVCBootstrap;

using mvcForum.Core;
using mvcForum.Core.Abstractions.Interfaces;
using mvcForum.Core.Specifications;
using mvcForum.Web.Controllers;
using mvcForum.Web.Extensions;
using mvcForum.Web.Helpers;
using mvcForum.Web.ViewModels;
using mvcForum.Web.ViewModels.Delete;
using mvcForum.Web.ViewModels.Update;
using mvcForum.Web.Interfaces;

namespace mvcForum.Web.Areas.Forum.Controllers {

	/// <summary>
	/// User Controller.
	/// The controller handles showing the user's public information.
	/// </summary>
	public class ProfileController : ThemedForumBaseController {
		private readonly IConfiguration config;

		#region Constructors
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="config">Injected IConfiguration instance.</param>
		public ProfileController(IWebUserProvider userProvider, IContext context, IConfiguration config)
			: base(userProvider, context) {

			// Store the configuration instance as a variable on the controller instance.
			this.config = config;
		}
		#endregion

		#region Read
		public ActionResult Index(Int32 id, String name) {
			try {
				// Get the user!
				ForumUser user = this.GetRepository<ForumUser>().Read(id);
				// Get the user's settings from the repository.
				//ForumUserSettings settings = this.GetRepository<ForumUserSettings>().ReadOne(new ForumUserSettingsSpecifications.SpecificUser(user));

				// Create the user model.
				UserViewModel model = new UserViewModel(user/*, settings*/);
				model.Path = new Dictionary<string, string>();

				return View(model);
			}
			catch { }

			// TODO: Handle the error, we need a common error page!!
			return View();
		}
		#endregion

		#region Update
		[Authorize]
		public ActionResult Update() {
			UpdateUserViewModel model = new UpdateUserViewModel(this.ActiveUser, this.User.Identity.Name != this.ActiveUser.Name);
			return View(model);
		}

		[Authorize]
		[HttpPost]
		public ActionResult Update(UpdateUserViewModel model) {
			if (ModelState.IsValid) {
				// Is this the actual current user?
				if (model.Id == this.ActiveUser.Id) {
					Boolean oldPasswordGiven = false;
					Boolean failed = false;
					// Did the user type in the existing password??
					if (!String.IsNullOrEmpty(model.OldPassword)) {
						// Yes, is it correct??
						oldPasswordGiven = Membership.Provider.ValidateUser(this.ControllerContext.RequestContext.HttpContext.User.Identity.Name, model.OldPassword);
					}

					// TODO: Use interface to do this!!!!!!
					MembershipUser u = Membership.Provider.GetUser(this.ControllerContext.RequestContext.HttpContext.User.Identity.Name, false);

					if (this.ActiveUser.EmailAddress != model.Email) {
						String existingUser = Membership.Provider.GetUserNameByEmail(model.Email);
						if (!String.IsNullOrWhiteSpace(existingUser)) {
							ModelState.AddModelError("Email", ForumHelper.GetString("EditProfile.ExistingEmail"));
							failed = true;
						}
					}

					// Was the e-mail address changed??
					if (this.ActiveUser.EmailAddress != model.Email) {
						// Yes, then the password is needed, was it given and correct?
						if (!this.ActiveUser.ExternalAccount && !oldPasswordGiven) {
							// No, let's stop then!
							ModelState.AddModelError("", ForumHelper.GetString("EditProfile.MissingPassword"));
							failed = true;
						}
						else {
							// Yes, e-mail address changed!
							u.Email = model.Email;
							// TODO: Send out e-mail?? To old and new?
						}
					}

					// TODO: Handled by the attributes on the model?
					if (!String.IsNullOrWhiteSpace(model.NewPassword) && model.NewPassword != model.RepeatedNewPassword) {
						ModelState.AddModelError("", ForumHelper.GetString("EditProfile.PasswordsDoesNotMatch"));
						failed = true;
					}
					if (!this.ActiveUser.ExternalAccount && !String.IsNullOrWhiteSpace(model.NewPassword) && model.NewPassword == model.RepeatedNewPassword) {
						if (!oldPasswordGiven) {
							ModelState.AddModelError("", ForumHelper.GetString("EditProfile.OldPasswordsRequired"));
							failed = true;
						}
						else {
							u.ChangePassword(model.OldPassword, model.NewPassword);
						}
					}
					if (!failed) {
						try {
							// TODO: Use interface to do this!!!!!!
							Membership.Provider.UpdateUser(u);

							ForumUser user = this.ForumUserRepository.ReadOne(new ForumUserSpecifications.SpecificProviderUserKey(u.ProviderUserKey.ToString()));
							// TODO: If null ?!?!?!?!?!?
							if (user != null) {
								user.Culture = model.Culture;
								user.Timezone = model.Timezone;
								user.FullName = model.FullName;
								user.UseFullName = !String.IsNullOrWhiteSpace(user.FullName);
								// TODO: User flag, use full name, etc etc etc.
							}

							this.Context.SaveChanges();

							// The profile was updated, let's tell the user!
							TempData.Add("Status", ForumHelper.GetString("EditUser.SettingsSaved"));
						}
						catch (FormatException) {
							ModelState.AddModelError("", ForumHelper.GetString("EditProfile.InvalidEmailAddress"));
						}
					}
				}
			}

			model = new UpdateUserViewModel(this.ActiveUser, this.User.Identity.Name != this.ActiveUser.Name);
			return View(model);
		}
		#endregion

		#region Delete
		[Authorize]
		public ActionResult Delete(Int32 id) {
			// TODO:
			DeleteUserViewModel model = new DeleteUserViewModel();
			return View(model);
		}

		[Authorize]
		[HttpPost]
		public ActionResult Delete(DeleteUserViewModel model) {
			if (model.Confirm) {
				// TODO:
				Membership.Provider.DeleteUser(this.ActiveUser.Name, true);
				// TODO:
				return Redirect("/");
			}
			return View(model);
		}
		#endregion
	}
}