﻿using System;
using System.Linq;
using System.Web.Mvc;
using System.Web.Security;

using ApplicationBoilerplate.DataProvider;

using MVCBootstrap.Web.Mvc.Extensions;
using MVCBootstrap.Web.Mvc.Interfaces;

using mvcForum.Core;
using mvcForum.Web.Controllers;
using ForumUserVM = mvcForum.Web.Areas.ForumAPI.ViewModels.ForumUser;
using System.Collections.Generic;
using mvcForum.Core.Specifications;

namespace mvcForum.Web.Areas.ForumAPI.Controllers {

	public class UserController : BaseAPIController {
		private readonly IMembershipService membership;

		public UserController(IMembershipService membership) {
			this.membership = membership;
		}

		[HttpPost]
		[Authorize(Roles = Constants.Roles.Combined)]
		public ActionResult Create(ForumUserVM model, String mode) {
			if (ModelState.IsValid) {
				MembershipCreateStatus createStatus = this.membership.CreateUser(model.Name, model.Password, model.EmailAddress);
				if (createStatus == MembershipCreateStatus.Success) {
					this.membership.UnlockUser(model.Name);
					return new HttpStatusCodeResult(200);
				}

				return new HttpStatusCodeResult(500, this.membership.ErrorCodeToString(createStatus, this.textManager));
			}

			return new HttpStatusCodeResult(500, ModelState.ErrorString());
		}

		[Authorize(Roles = Constants.Roles.Combined)]
		public ActionResult Read(Int32 id, String mode) {
			// TODO: Access!
			ForumUser user = this.GetRepository<ForumUser>().Read(id);
			//MVCBootstrap.User u = this.GetRepository<MVCBootstrap.User>().ReadOne(new MVCBootstrap.UserSpecifications.SpecificProviderUserKey(new Guid(user.ProviderId)));
			MembershipUser u = Membership.GetUser(Membership.GetUserNameByEmail(user.EmailAddress));

			return this.CustomJson(
				new ForumUserVM {
					Id = user.Id,
					Name = user.Name,
					EmailAddress = user.EmailAddress,
					Deleted = user.Deleted,
					FirstVisit = user.FirstVisit,
					LastVisit = user.LastVisit,
					LastIP = user.LastIP,
					Active = user.Active,
					Locked = u.IsLockedOut
				});
		}

		[HttpPost]
		[Authorize(Roles = Constants.Roles.Combined)]
		public ActionResult Update(ForumUserVM model, String mode) {
			if (ModelState.IsValid) {
				// TODO: Access!!
				ForumUser user = this.GetRepository<ForumUser>().Read(model.Id);
				user.Deleted = model.Deleted;

				MembershipUser u = Membership.Provider.GetUser(user.Name, false);
				u.Email = model.EmailAddress;
				Membership.Provider.UpdateUser(u);

				user.Active = model.Active;

				// TODO: Hack? Maybe move an active bit to ForumUser?
				//if ((u.IsLockedOut && !model.Active) || (!u.IsLockedOut && model.Active)) {
				//    MVCBootstrap.User usr = this.GetRepository<MVCBootstrap.User>().Read((Guid)u.ProviderUserKey);
				//    usr.Locked = !model.Active;
				//}

				this.Context.SaveChanges();

				return new HttpStatusCodeResult(200);
			}

			return new HttpStatusCodeResult(500, ModelState.ErrorString());
		}

		[HttpPost]
		[Authorize(Roles = Constants.Roles.SolutionAdmin)]
		public ActionResult UpdateRoles(Int32 id, String roles, String mode) {
			ForumUser user = this.GetRepository<ForumUser>().Read(id);
			// TODO: Guid ?? Can we count on that?
			MembershipUser mu = Membership.GetUser((Object)user.ProviderId);
			String[] existingRoles = Roles.GetRolesForUser(mu.UserName);
			if (String.IsNullOrWhiteSpace(roles) && existingRoles.Any()) {
				Roles.RemoveUserFromRoles(mu.UserName, existingRoles);
			}
			else if (!String.IsNullOrWhiteSpace(roles)) {
				String[] roleArray = roles.Split(new String[] { "," }, StringSplitOptions.RemoveEmptyEntries);
				String[] removeRoles = existingRoles.Where(r => roleArray.Contains(r) == false).ToArray();
				String[] newRoles = roleArray.Where(r => existingRoles.Contains(r) == false).ToArray();

				if (removeRoles.Any()) {
					Roles.RemoveUserFromRoles(mu.UserName, removeRoles);
				}
				if (newRoles.Any()) {
					Roles.AddUserToRoles(mu.UserName, newRoles);
				}
			}

			return new HttpStatusCodeResult(200);
		}

		[HttpPost]
		[Authorize(Roles = Constants.Roles.Combined)]
		public ActionResult Unlock(Int32 id, String mode) {
			// TODO: Access
			IRepository<ForumUser> userRepo = this.GetRepository<ForumUser>();
			ForumUser user = userRepo.Read(id);

			//MembershipUser mu = Membership.GetUser(new Guid(user.ProviderId));
			// We'll delete the MemberShipUser, no more logging in!
			Membership.Provider.UnlockUser(user.Name);

			return new HttpStatusCodeResult(200);
		}

		[HttpPost]
		[Authorize(Roles = Constants.Roles.Combined)]
		public ActionResult Delete(Int32 id, String mode) {
			// TODO: Access
			IRepository<ForumUser> userRepo = this.GetRepository<ForumUser>();
			ForumUser user = userRepo.Read(id);

			// Mark the user as deleted!
			user.Deleted = true;

			//MembershipUser mu = Membership.GetUser(new Guid(user.ProviderId));
			// We'll delete the MemberShipUser, no more logging in!
			Membership.DeleteUser(user.Name);

			this.Context.SaveChanges();

			return new HttpStatusCodeResult(200);
		}

		[Authorize(Roles = Constants.Roles.Combined)]
		public ActionResult Search(String query, String mode) {
			IEnumerable<ForumUser> results = new List<ForumUser>();
			if (!String.IsNullOrWhiteSpace(query)) {
				results = this.ForumUserRepository.ReadMany(new ForumUserSpecifications.Search(query));
			}

			return this.GetUserList(results, results.Count(), 0);
		}

		private ContentResult GetUserList(IEnumerable<ForumUser> users, Int32 pageSize, Int32 skipped) {
			Int32 count = users.Count();
			return this.CustomJson(new {
				Count = count,
				PageSize = pageSize,
				Skipped = skipped,
				Users = users
					.Skip(skipped)
					.Take(pageSize)
					.Select(u => new ForumUserVM {
						Id = u.Id,
						Name = u.Name,
						EmailAddress = u.EmailAddress,
						FirstVisit = u.FirstVisit,
						Deleted = u.Deleted,
						Active = u.Active,
						ApiUrl = Url.Action("read", "user", new { mode = "json", area = "forumapi", id = u.Id })
					}).OrderBy(u => u.Name),
				Next = ((skipped + pageSize) < count ? Url.Action("list", "user", new { area = "forumapi", mode = "json", skip = (skipped + pageSize) }) : String.Empty),
				Previous = skipped > 0 ? Url.Action("list", "user", new { area = "forumapi", mode = "json", skip = (skipped - pageSize) > 0 ? skipped - pageSize : 0 }) : String.Empty
			});
		}

		[Authorize(Roles = Constants.Roles.Combined)]
		public ActionResult List(Int32? skip, String mode) {
			Int32 skipCount = 0;
			Int32 returnCount = 15;
			if (skip.HasValue && skip.Value > 0) {
				skipCount = skip.Value;
			}

			return this.GetUserList(this.ForumUserRepository.ReadAll().OrderByDescending(u => u.FirstVisit), returnCount, skipCount);
		}

		public ActionResult RoleList(Int32 id, String mode) {
			ForumUser fu = this.GetRepository<ForumUser>().Read(id);
			//MembershipUser user = Membership.GetUser((Object)fu.ProviderId);
			String[] roles = Roles.Provider.GetRolesForUser(fu.Name);

			return this.CustomJson(new { Roles = roles });
		}
	}
}